/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * AtomOfMoleculeSelectionDescriptor.cpp
 *
 *  Created on: Dec 11, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "Descriptors/AtomOfMoleculeSelectionDescriptor.hpp"
#include "Descriptors/AtomOfMoleculeSelectionDescriptor_impl.hpp"

#include "molecule.hpp"

#include "Helpers/helpers.hpp"


AtomOfMoleculeSelectionDescriptor_impl::AtomOfMoleculeSelectionDescriptor_impl(){}

AtomOfMoleculeSelectionDescriptor_impl::~AtomOfMoleculeSelectionDescriptor_impl(){}

bool AtomOfMoleculeSelectionDescriptor_impl::predicate(std::pair<atomId_t,const atom*> atom) const{
  return World::getInstance().isSelected(atom.second->getMolecule());
}

atom* AtomOfMoleculeSelectionDescriptor_impl::find(){
  World::MoleculeSet &set = getSelectedMolecules();
  for (World::MoleculeSet::internal_iterator iter = set.begin_internal();
      iter != set.end_internal();
      ++iter) {
    if (iter->second->begin() != iter->second->end())
      return *(iter->second->begin());
  }
  return 0;
}

const atom* AtomOfMoleculeSelectionDescriptor_impl::find() const {
  const World::MoleculeSet &set = getSelectedMolecules();
  for (World::MoleculeSet::const_iterator iter = set.begin();
      iter != set.end();
      ++iter) {
    if (iter->second->begin() != iter->second->end())
      return *(iter->second->begin());
  }
  return 0;
}

std::vector<atom*> AtomOfMoleculeSelectionDescriptor_impl::findAll(){
  std::vector<atom*> res;
  World::MoleculeSet &set = getSelectedMolecules();
  for (World::MoleculeSet::internal_iterator iter = set.begin_internal();
      iter != set.end_internal();
      ++iter) {
    std::copy(iter->second->begin(), iter->second->end(), res.begin());
  }
  return res;
}

std::vector<const atom*> AtomOfMoleculeSelectionDescriptor_impl::findAll() const{
  std::vector<const atom*> res;
  const World::MoleculeSet &set = getSelectedMolecules();
  for (World::MoleculeSet::const_iterator iter = set.begin();
      iter != set.end();
      ++iter) {
    std::copy(iter->second->begin(), iter->second->end(), res.begin());
  }
  return res;
}

World::MoleculeSet& AtomOfMoleculeSelectionDescriptor_impl::getSelectedMolecules(){
  return World::getInstance().selectedMolecules;
}

const World::MoleculeSet& AtomOfMoleculeSelectionDescriptor_impl::getSelectedMolecules() const{
  return World::getInstance().selectedMolecules;
}

AtomDescriptor AtomsByMoleculeSelection(){
  return AtomDescriptor(AtomDescriptor::impl_ptr(new AtomOfMoleculeSelectionDescriptor_impl()));
}
