/*
 * Project: MoleCuilder
 * Description: creates and alters _atomecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * AtomOrderDescriptor.cpp
 *
 *  Created on: Mar 22, 2012
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "AtomOrderDescriptor.hpp"
#include "AtomOrderDescriptor_impl.hpp"

#include "Atom/atom.hpp"
#include "CodePatterns/Observer/ObservedContainer_impl.hpp"

using namespace std;


AtomOrderDescriptor_impl::AtomOrderDescriptor_impl(int _id) :
  id(_id)
{}

AtomOrderDescriptor_impl::~AtomOrderDescriptor_impl()
{}

bool AtomOrderDescriptor_impl::predicate(std::pair<atomId_t,const atom*> _atom) const
{
  const atom *Walker = find();
  return (Walker == _atom.second);
}

AtomDescriptor AtomByOrder(int id){
  return AtomDescriptor(AtomDescriptor::impl_ptr(new AtomOrderDescriptor_impl(id)));
}

World::AtomSet& AtomOrderDescriptor_impl::getAtoms()
{
  return World::getInstance().atoms;
}

const World::AtomSet& AtomOrderDescriptor_impl::getAtoms() const
{
  return const_cast<const World &>(World::getInstance()).atoms;
}

atom *AtomOrderDescriptor_impl::find(){
  World::AtomSet &atoms = getAtoms();

  int i=0;
  atom *_atom = NULL;
  if (id == 0) {
    return NULL;
  } else if (id > 0) {
    World::AtomSet::internal_iterator res = atoms.begin_internal();
    for (; res != atoms.end_internal(); ++res) { // when iterator is normal, ++ goes forward!
      ++i;
      if (id == i) {
        _atom = res->second;
        break;
      }
    }
  } else {
    World::AtomSet::reverse_internal_iterator res = atoms.rbegin_internal();
    for (; res != atoms.rend_internal(); ++res) {  // when iterator is reverse, ++ goes backward!
      --i;
      if (id == i) {
        _atom = res->second;
        break;
      }
    }
  }

  return _atom;
}

const atom *AtomOrderDescriptor_impl::find() const {
  const World::AtomSet &atoms = getAtoms();

  int i=0;
  const atom *_atom = NULL;
  if (id == 0) {
    return NULL;
  } else if (id > 0) {
    World::AtomSet::const_iterator res = atoms.begin();
    for (; res != atoms.end(); ++res) { // when iterator is normal, ++ goes forward!
      ++i;
      if (id == i) {
        _atom = res->second;
        break;
      }
    }
  } else {
    World::AtomSet::const_reverse_iterator res = atoms.rbegin();
    for (; res != atoms.rend(); ++res) {  // when iterator is reverse, ++ goes backward!
      --i;
      if (id == i) {
        _atom = res->second;
        break;
      }
    }
  }

  return _atom;
}

vector<atom*> AtomOrderDescriptor_impl::findAll(){
  atom *res = find();
  return (res)?(vector<atom*>(1,res)):(vector<atom*>());
}

vector<const atom*> AtomOrderDescriptor_impl::findAll() const {
  const atom *res = find();
  return (res)?(vector<const atom*>(1,res)):(vector<const atom*>());
}
